//
//  TESTTextfieldView.m
//
//  Created by Thomas Wolters on 20/10/15.
//  Copyright © 2015 fluodmobile. All rights reserved.
//

#import "Demo3HighwayToHellView.h"
#import "TPKeyboardAvoidingScrollView.h"


@interface Demo3HighwayToHellView() <UITextFieldDelegate>

@property (nonatomic, strong) UITextField* textField;
@property (nonatomic, strong) UIButton* button;
@property (nonatomic, strong) CAEmitterLayer* fireEmitter;
@property (nonatomic, assign) int level;
@property (nonatomic, strong) TPKeyboardAvoidingScrollView* scrollView;
@end

@implementation Demo3HighwayToHellView

- (instancetype)init{
    self = [super init];
    if (!self) {
        return nil;
    }
    self.scrollView = [TPKeyboardAvoidingScrollView new];
    self.scrollView.userInteractionEnabled = YES;
    UITapGestureRecognizer* tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(actionHideKeyboard:)];
    [self.scrollView addGestureRecognizer:tapGesture];
    [self addSubview:self.scrollView];
    
    self.backgroundColor = [UIColor darkGrayColor];
    
    self.textField = [UITextField new];
    self.textField.backgroundColor = [UIColor lightGrayColor];
    self.textField.secureTextEntry = YES;
    self.textField.layer.cornerRadius = 5;
    [self.scrollView addSubview:self.textField];
    
    self.button = [UIButton new];
    [self.button setTitle:@"Verify" forState:UIControlStateNormal];
    [self.button addTarget:self action:@selector(actionTap:) forControlEvents:UIControlEventTouchUpInside];
    [self.button setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    self.button.backgroundColor = [UIColor whiteColor];
    self.button.layer.cornerRadius = 5;
    self.button.layer.borderColor = [UIColor lightGrayColor].CGColor;
    self.button.layer.borderWidth = 2;
    [self.scrollView addSubview:self.button];
    
    self.textField.placeholder = @"Password";
    
    return self;
}

-(void)layoutSubviews{
    [super layoutSubviews];
    self.scrollView.frame = self.bounds;
    self.textField.frame = [self textFieldFrame];
    self.button.frame = (CGRect){50,self.bounds.size.height-120,self.bounds.size.width-100,50};
}

-(CGRect)textFieldFrame{
    return (CGRect){50,self.bounds.size.height-180,self.bounds.size.width-100,50};
}

-(void)actionTap:(id)sender{
    if ([self.textField.text isEqualToString:@"0000"]){
        self.level = 0;
        self.textField.text = @"";
        [self.fireEmitter setValue:[NSNumber numberWithInt:0] forKeyPath:@"emitterCells.fire.birthRate"];
    }
    else{
        [self actionHideKeyboard:nil];
        if (self.level <6){
            self.level = 5;
        }
        self.level+=3;

        [self animationBounceForView:self.textField];
        [self setSmokeLevel:self.level];
        
    }
}

-(void)animationBounceForView:(UIView*)view{
    //update view frame
    CGRect frame = self.textField.frame;
    frame.origin.x -=4;
    frame.origin.y -=1;
    frame.size.width -=1;
    frame.size.height -=1;
    self.textField.frame = frame;
    
    [UIView animateWithDuration:1.0f delay:0.0f usingSpringWithDamping:.001f initialSpringVelocity:10000.0 options:UIViewAnimationOptionLayoutSubviews animations:^{
        view.frame = [self textFieldFrame];
    } completion:^(BOOL complete){
        [self setNeedsLayout];
    }];
}

-(void)actionHideKeyboard:(id)sender{
    [self.textField resignFirstResponder];
}

-(void)setSmokeLevel:(float)level{
    if (!self.fireEmitter){
        // Create the emitter layers
        self.fireEmitter	= [CAEmitterLayer layer];

        CGPoint emitterposition = CGPointMake(self.textField.frame.origin.x+self.textField.frame.size.width/2,self.textField.frame.origin.y);
        CGSize emitterSize = CGSizeMake(self.textField.frame.size.width*0.8, 0);
        
        self.fireEmitter.emitterPosition = emitterposition;
        self.fireEmitter.emitterSize	= emitterSize;
        self.fireEmitter.emitterMode	= kCAEmitterLayerOutline;
        self.fireEmitter.emitterShape	= kCAEmitterLayerLine;
        self.fireEmitter.emitterCells	= [NSArray arrayWithObject:[self cellFire]];
        self.fireEmitter.frame = self.bounds;
        
        [self.scrollView.layer addSublayer:self.fireEmitter];
    }
    
    float zeroToOne = level/20.0;

    [self.fireEmitter setValue:[NSNumber numberWithInt:(zeroToOne * 500)] forKeyPath:@"emitterCells.fire.birthRate"];
}

-(CAEmitterCell*)cellFire{
    CAEmitterCell* fire = [CAEmitterCell emitterCell];
    [fire setName:@"fire"];
    fire.birthRate			= 100;
    fire.emissionLongitude  = M_PI;
    fire.velocity			= -80;
    fire.yAcceleration		= -200;
    fire.lifetime			= 50;
    fire.color = [[UIColor colorWithRed:0.8 green:0.4 blue:0.2 alpha:0.1] CGColor];
    fire.contents = (id) [[UIImage imageNamed:@"fire"] CGImage];
    return fire;
}

@end