//
//  TESTProgressView.m
//
//  Created by Thomas Wolters on 19/10/15.
//  Copyright © 2015 fluodmobile. All rights reserved.
//
#import "Demo5AnimationLayerReplicatorView.h"

#define DURATION 0.5

#define INIT_scaleX 1.0f
#define INIT_instanceCount 31
#define INIT_cornerRadius 4
#define INIT_instanceDelay 0.2f
#define INIT_elementDimension 8
#define INIT_elementAngle 0.2f
#define INIT_transitionX 0
#define INIT_transitionY 0
#define INIT_transitionZ 1

@interface Demo5AnimationLayerReplicatorView()
@property (nonatomic, strong) CAReplicatorLayer* layerReplicator;
@property (nonatomic, strong) CALayer* layerSeed;
@property (nonatomic, assign) int iteration;
@property (nonatomic, strong) UITableView* tableView;
@property (nonatomic, assign) TESTAnimationScenario currentScenario;

@end

@implementation Demo5AnimationLayerReplicatorView
- (instancetype)init{
    self = [super init];
    if (!self) {
        return nil;
    }
    self.backgroundColor = [UIColor lightGrayColor];
    
    self.layerReplicator = [CAReplicatorLayer new];

    self.layerSeed = [CALayer new];
    self.layerSeed.backgroundColor = [UIColor blackColor].CGColor;
    self.layerSeed.borderColor = [UIColor whiteColor].CGColor;
    self.layerSeed.borderWidth = 1;
    self.layerSeed.cornerRadius = 2;

    [self.layerReplicator addSublayer:self.layerSeed];
    [self setInstanceCount:INIT_instanceCount];
    self.layerReplicator.backgroundColor = [UIColor lightGrayColor].CGColor;
    [self updateTransition];
    self.instanceDelay = INIT_instanceDelay;

    [self.layer addSublayer:self.layerReplicator];
    
    self.tableView = [UITableView new];
    self.tableView.backgroundColor = [UIColor lightGrayColor];
    [self addSubview:self.tableView];
    
    self.scaleX = INIT_scaleX;
    self.elementDimension = INIT_elementDimension;
    self.elementAngle = INIT_elementAngle;
    self.transitionX = INIT_transitionX;
    self.transitionY = INIT_transitionY;
    self.transitionZ = INIT_transitionZ;
    return self;
}

-(void)updateTransition{
    self.layerReplicator.instanceTransform = CATransform3DMakeRotation(self.elementAngle, self.transitionX, self.transitionY, self.transitionZ);
    [self setNeedsLayout];
}

-(void)setCornerRadius:(float)cornerRadius{
    _cornerRadius = cornerRadius;
    [self setNeedsLayout];
}
-(void)setElementAngle:(float)elementAngle{
    _elementAngle = elementAngle;
    [self updateTransition];
}

-(void)setTransitionX:(float)transitionX{
    _transitionX = transitionX;
    [self updateTransition];
}

-(void)setTransitionY:(float)transitionY{
    _transitionY = transitionY;
    [self updateTransition];
}

-(void)setTransitionZ:(float)transitionZ{
    _transitionZ = transitionZ;
    [self updateTransition];
}

-(void)setInstanceCount:(float)instanceCount{
    _instanceCount = instanceCount;
    self.layerReplicator.instanceCount = self.instanceCount;
}

-(void)setInstanceDelay:(float)instanceDelay{
    _instanceDelay = instanceDelay;
    self.layerReplicator.instanceDelay = self.instanceDelay;
}

-(void)setElementDimension:(float)elementDimension{
    _elementDimension = elementDimension;
    [self setNeedsLayout];
}

-(void)layoutSubviews{
    [super layoutSubviews];
    self.layerReplicator.frame = (CGRect){self.bounds.size.width/4,0,self.bounds.size.width/2,self.bounds.size.height/2};
    self.layerSeed.frame = (CGRect) {0,self.layerReplicator.frame.size.height/2,self.elementDimension*2,self.elementDimension};
    self.layerSeed.cornerRadius = self.cornerRadius;
    
    float tableViewOriginY = self.layerReplicator.frame.origin.y+self.layerReplicator.frame.size.height+10;
    self.tableView.frame = (CGRect){0,tableViewOriginY,self.bounds.size.width,self.bounds.size.height-tableViewOriginY};
}

-(void)updateProgress:(float)progress text:(NSString*)text{
    NSLog(@"update");
    
    self.iteration = self.iteration % self.layerReplicator.instanceCount;
    self.iteration = self.iteration % 5;
    UIColor* colorTo;
    if (self.iteration == 0){
        colorTo = [UIColor whiteColor];
    }
    else{
        colorTo = [UIColor blackColor]; // [Demo5AnimationLayerReplicatorView disco];
    }
    
    CABasicAnimation* animation = [[CABasicAnimation alloc] init];
    animation.keyPath = @"backgroundColor";
    //    animation.fromValue = (id)[TESTProgressView disco].CGColor;
    animation.toValue = (id)colorTo.CGColor;
    animation.duration = DURATION/2;
    
    animation.removedOnCompletion = NO;
    animation.fillMode = kCAFillModeForwards;
    [self.layerSeed addAnimation:animation forKey:nil];
    
    switch (self.currentScenario) {
        case TESTAnimationScenarioFlower:{
            CABasicAnimation* animation3 = [[CABasicAnimation alloc] init];
            animation3.keyPath = @"transform";
            animation3.toValue = [NSValue valueWithCATransform3D:CATransform3DMakeTranslation(20*self.iteration, 0, 0)];
            animation3.duration = DURATION*2;
            animation3.removedOnCompletion = NO;
            animation3.fillMode = kCAFillModeForwards;
            [self.layerSeed addAnimation:animation3 forKey:nil];
        }
            break;
        case TESTAnimationScenarioPulse:{

            
        }
            break;
        default:
            break;
    }

    CABasicAnimation* animation2 = [[CABasicAnimation alloc] init];
    animation2.keyPath = @"transform.scale.x";

    if (self.iteration  == 0){
        animation2.toValue = @(self.scaleX);
    }
    else{
        animation2.toValue = @(1);
    }
    animation2.duration = DURATION;
    animation2.removedOnCompletion = NO;
    animation2.fillMode = kCAFillModeForwards;
    [self.layerSeed addAnimation:animation2 forKey:nil];
    
    self.iteration++;
}

-(void)resetValues{
    self.scaleX = INIT_scaleX;
    [self setInstanceCount:INIT_instanceCount];
    self.cornerRadius = INIT_cornerRadius;
    [self setInstanceDelay:INIT_instanceDelay];
    self.elementDimension = INIT_elementDimension;
    self.elementAngle = INIT_elementAngle;
    self.transitionX = INIT_transitionX;
    self.transitionY = INIT_transitionY;
    self.transitionZ = INIT_transitionZ;
}

-(void)loadScenarioPulseValues{
    self.scaleX = 10.0f;
}

-(void)setAnimationScenario:(TESTAnimationScenario)scenario{
    self.currentScenario = scenario;
    self.instanceCount = 53;
    self.elementAngle = 0.1;
    
    [self resetValues];
    if (scenario == TESTAnimationScenarioPulse){
        [self loadScenarioPulseValues];
    }
    [self setNeedsLayout];
    [self updateTransition];
}

+ (UIColor*)disco {
    CGFloat hue = arc4random() % 256 / 256.0;    //  0.0 to 1.0
    CGFloat saturation = (arc4random() % 128 / 256.0) + 0.5;    //  0.5 to 1.0, away from white
    CGFloat brightness = (arc4random() % 128 / 256.0) + 0.5;    //  0.5 to 1.0, away from black
    return [UIColor colorWithHue:hue saturation:saturation brightness:brightness alpha:1];
}

@end
